%define build_ocamlopt	1
%ifarch ppc64 %mips %arm
%define build_ocamlopt	0
%endif
%define build_labltk	1
%define major	4.00
%define minor	0

Name:		ocaml
Version:	%{major}.%{minor}
Release:	1
Summary:	The Objective Caml compiler and programming environment
URL:		http://caml.inria.fr
License:	QPL with exceptions and LGPLv2 with exceptions
Group:		Development/Other
Source0:	http://caml.inria.fr/pub/distrib/ocaml-%{major}/%{name}-%{version}.tar.bz2
Source1:	http://caml.inria.fr/pub/distrib/ocaml-%{major}/%{name}-%{major}-refman-html.tar.gz

# IMPORTANT:
# The contents (META's files) of this tarball comes from findlib
# This tarball have to be updated when findlib or ocaml are updated to a new version
# these META's files are only generated by the ./configure in the findlib source directory
# (camlp4 and ocaml-labltk have to be installed for this operation)
# steps:
# get findlib sources from http://projects.camlcity.org/projects/findlib.html
# extract, run ./configure
# then,
# tar -Jcf findlib-1.2.7-ocaml-%{version}-meta-files.tar.xz  site-lib-src/*/META
# notice the version macro in filename, this to imply that the META files has
# to be built for *THIS* specific version!
Source5:	findlib-1.3.3-ocaml-%{version}-meta-files.tar.xz

Patch0:         ocaml-3.12.0-rpath.patch
Patch1:         ocaml-user-cflags.patch
Patch3:		ocaml-3.11.0-ocamltags-no-site-start.patch
Patch7:		ocaml-3.11.0-no-opt-for-debug-and-profile.patch
Patch8:		ocaml-3.04-larger-buffer-for-uncaught-exception-messages.patch
Patch9:		ocaml-4.00.0-handle-tk-8.6.patch
Patch16:	ocaml-4.00.0-lib64.patch
Patch17:	ocaml-3.11.0-db52.patch
Patch18:	ocaml-3.09.3-compile-emacs-files-in-build-dir.patch

BuildRequires:	libx11-devel
BuildRequires:	ncurses-devel
BuildRequires:	tcl
BuildRequires:	tcl-devel
BuildRequires:	tk
BuildRequires:	tk-devel
BuildRequires:	emacs
BuildRequires:	db-devel

%rename		ocaml-emacs

%package	doc
Summary:	Documentation for OCaml
Group:		Books/Computer books
Requires:	%{name} = %{version}

%package -n	camlp4
Summary:	Preprocessor for OCaml
Group:		Development/Other
Requires:	%{name} = %{version}

%package	labltk
Summary:	Tk toolkit binding for OCaml
Group:		Development/Other
Requires:	%{name} = %{version}
Requires:	tk-devel
Obsoletes:	ocamltk

%package	sources
Summary:	OCaml sources
Group:		Development/Other
# don't add crazy deps
AutoReqProv:	No

%description
Objective Caml is a high-level, strongly-typed, functional and object-oriented
programming language from the ML family of languages.

This package comprises two batch compilers (a fast bytecode compiler and an
optimizing native-code compiler), an interactive toplevel system, Lex&Yacc
tools, a replay debugger, and a comprehensive library.

%description	doc
Documentation for OCaml

%description -n	camlp4
Preprocessor for OCaml

%description	labltk
Tk toolkit binding for OCaml

%description	sources
OCaml sources

%prep
%setup -q -T -b 0
%setup -q -T -D -a 1
%setup -q -T -D -a 5
%patch0 -p1 -b .rpath
%patch1 -p1 -b .cflags
%patch3 -p1
%patch7 -p1
%patch8 -p1
%patch9 -p1 -b .tk
%patch16 -p1 -b .lib64
#patch17 -p1 -b .db4
#patch18 -p1 -b .emacs

rm -rf `find -name .cvsignore`

# fix incorrect reference in camlp4 META file
sed -ri -e 's|directory = "/usr/lib.*/ocaml/camlp4"|directory = "%{_libdir}/ocaml/camlp4"|g' site-lib-src/camlp4/META

%build
%ifarch alpha
echo %{optflags} | grep -q mieee || { echo "on alpha you need -mieee to compile ocaml"; exit 1; }
%endif

CFLAGS="$RPM_OPT_FLAGS" ./configure \
    -bindir %{_bindir} \
    -libdir %{_libdir}/ocaml \
    -x11lib %{_libdir} \
    -x11include %{_includedir} \
    -mandir %{_mandir}/man1

make world
%if %{build_ocamlopt}
make opt opt.opt
%endif
make -C emacs ocamltags


%install
make install BINDIR=%{buildroot}%{_bindir} LIBDIR=%{buildroot}%{_libdir}/ocaml MANDIR=%{buildroot}%{_mandir}

# remove stupid camlp4o.opt which can't work
#rm -f %{buildroot}%{_bindir}/camlp4*.opt
#rm -f %{buildroot}%{_mandir}/man1/camlp4*.opt.*

cd emacs; make install install-ocamltags BINDIR=%{buildroot}%{_bindir} EMACSDIR=%{buildroot}%{_datadir}/emacs/site-lisp; cd -

# fix
perl -pi -e "s|%{buildroot}||" %{buildroot}%{_libdir}/ocaml/ld.conf

%if %{build_ocamlopt}
# only keep the binary versions, which are much faster, except for camlp4
# as native code cannot do a dynamic load
for i in %{buildroot}%{_bindir}/*.opt ; do
  [[ $i == %{buildroot}%{_bindir}/camlp4* ]] && continue
  ln -sf `basename $i` ${i%.opt}
done
%endif


install -d %{buildroot}%{_sysconfdir}/emacs/site-start.d
cat <<EOF >%{buildroot}%{_sysconfdir}/emacs/site-start.d/%{name}.el
(require 'caml-font)
(autoload 'caml-mode "caml" "Caml editing mode" t)
(add-to-list 'auto-mode-alist '("\\\\.mli?$" . caml-mode))
EOF

# don't package mano man pages since we have the html files
rm -rf %{buildroot}%{_mandir}/mano

# install findlib META files
cp -pr site-lib-src/* %{buildroot}%{_libdir}/ocaml/

rm -f %{name}.list
n="labltk|camlp4|ocamlbrowser|tkanim"
(cd %{buildroot} ; find usr/bin ! -type d -printf "/%%p\n" | egrep -v $n) >> %{name}.list
(cd %{buildroot} ; find usr/%{_lib}/ocaml ! -type d -printf "/%%p\n" | egrep -v $n) >> %{name}.list
(cd %{buildroot} ; find usr/%{_lib}/ocaml   -type d -printf "%%%%dir /%%p\n" | egrep -v $n) >> %{name}.list

# install sources
install -d -m 755 %{buildroot}%{_prefix}/src
tar xvjf %{SOURCE0} -C %{buildroot}%{_prefix}/src
mv %{buildroot}%{_prefix}/src/%{name}-%{version} %{buildroot}%{_prefix}/src/%{name}
install -d %{buildroot}%{_includedir}
ln -s %{_libdir}/ocaml/caml %{buildroot}%{_includedir}/

%files -f %{name}.list
%doc Changes LICENSE README
%{_includedir}/caml
%{_mandir}/man1/*
%{_datadir}/emacs/site-lisp/*
%config(noreplace) %{_sysconfdir}/emacs/site-start.d/*

%files doc
%doc htmlman/* 
%if %build_ocamlopt
%{_mandir}/man3/*
%endif

%if %{build_labltk}
%files labltk
%doc otherlibs/labltk/README otherlibs/labltk/example*
%{_bindir}/*labltk*
%{_bindir}/ocamlbrowser
%{_libdir}/ocaml/*labltk*
%{_libdir}/ocaml/stublibs/dlllabltk.so
%endif

%files -n camlp4
%{_bindir}/*camlp4*
%{_libdir}/ocaml/camlp4

%files sources
%{_prefix}/src/%{name}


%changelog
* Sat Sep 29 2012 Bernhard Rosenkraenzer <bero@bero.eu> 4.00.0-1
+ Revision: 817960
- ocaml 4.00.0, findlib 1.3.3

* Sat May 12 2012 Crispin Boylan <crisb@mandriva.org> 3.12.1-10
+ Revision: 798408
- Rebuild with db5.3
- Rebuild

* Mon Jan 16 2012 Bernhard Rosenkraenzer <bero@bero.eu> 3.12.1-7
+ Revision: 761806
- Use db 5.2, drop requirement of legacy fb 5.1

* Fri Sep 16 2011 Alexandre Lissy <alissy@mandriva.com> 3.12.1-6
+ Revision: 700003
- Changing camlp4 META libdir to fix bug 64264
- Updating sources to ocaml 3.12.1
  Updating findlib meta files for findlib 1.2.7 and ocaml 3.12.1
  Removing merged patch for amd64 issue
  Enhancing findlib tarball comment for dumb people like me.

* Mon May 30 2011 Alexandre Lissy <alissy@mandriva.com> 3.12.0-5
+ Revision: 681835
- Adding upstream patch fixing bad interactions between ocaml and binutils 2.21 (see http://caml.inria.fr/mantis/view.php?id=5237)

* Sun Apr 10 2011 Per Øyvind Karlsen <peroyvind@mandriva.org> 3.12.0-4
+ Revision: 652367
- update meta files

* Wed Mar 30 2011 Per Øyvind Karlsen <peroyvind@mandriva.org> 3.12.0-3
+ Revision: 649267
- use %%rename
- cosmetics
- drop deprecated/obsolete/redundant %%mkrel, %%defattr, buildroot, %%clean etc.

* Thu Jan 27 2011 Per Øyvind Karlsen <peroyvind@mandriva.org> 3.12.0-2
+ Revision: 633196
- drop ancient debian menu (this package really needs some cleaning..)
- add a /usr/include/caml symlink for headers
- sync fedora cflags & rpath patches
- build against berkeley db 5.1

* Sat Aug 28 2010 Florent Monnier <blue_prawn@mandriva.org> 3.12.0-1mdv2011.0
+ Revision: 573767
- updated to last version 3.12.0

* Mon Aug 23 2010 Florent Monnier <blue_prawn@mandriva.org> 3.11.2-4mdv2011.0
+ Revision: 572373
- updated META files provided by findlib version 1.2.6

* Mon Mar 01 2010 Guillaume Rousse <guillomovitch@mandriva.org> 3.11.2-3mdv2010.1
+ Revision: 512981
- fix camlp4 META file correctly

* Mon Mar 01 2010 Guillaume Rousse <guillomovitch@mandriva.org> 3.11.2-2mdv2010.1
+ Revision: 512910
- emacs is insufficient to compile bytecode, switch to full emacs
- fix incorrect META file for camlp4 on x86_64
- new version

* Fri Jan 01 2010 Oden Eriksson <oeriksson@mandriva.com> 3.11.1-5mdv2010.1
+ Revision: 484726
- rebuilt against bdb 4.8

* Sun Sep 27 2009 Olivier Blin <blino@mandriva.org> 3.11.1-4mdv2010.0
+ Revision: 450174
- disable ocaml.opt* on mips & arm, not supported
  (from Arnaud Patard)

* Fri Sep 11 2009 Florent Monnier <blue_prawn@mandriva.org> 3.11.1-3mdv2010.0
+ Revision: 438520
- restore the previous solution to manage META files (sync for the tarball)
- restore the previous solution to manage META files

* Wed Sep 09 2009 Florent Monnier <blue_prawn@mandriva.org> 3.11.1-2mdv2010.0
+ Revision: 435818
- this tarball was not updated when ocaml and ocaml-findlib was updated,
  and there are currently things broken with this old tarball,
  instead of updating it, I think it's better to keep it in findlib where it's easier to keep it maintained

* Thu Jun 25 2009 Frederik Himpe <fhimpe@mandriva.org> 3.11.1-1mdv2010.0
+ Revision: 389199
- update to new version 3.11.1

* Wed Dec 24 2008 Guillaume Rousse <guillomovitch@mandriva.org> 3.11.0-2mdv2009.1
+ Revision: 318162
- rediff fuzzy patches
- site-lib hierarchy doesn't exist anymore

* Mon Dec 08 2008 Pixel <pixel@mandriva.com> 3.11.0-1mdv2009.1
+ Revision: 311906
- 3.11.0
- rediff/adapt patch9 (tk)

* Sat Dec 06 2008 Adam Williamson <awilliamson@mandriva.org> 3.10.2-5mdv2009.1
+ Revision: 310971
- rebuild for new tcl
- update tcl handling patch for 8.6
- new license policy

* Mon Sep 15 2008 Guillaume Rousse <guillomovitch@mandriva.org> 3.10.2-4mdv2009.0
+ Revision: 284872
- keep both bytecode and native versions for camlp4, as native ones have troubles with dynamic loading

* Mon Sep 01 2008 Guillaume Rousse <guillomovitch@mandriva.org> 3.10.2-3mdv2009.0
+ Revision: 278643
- re-enable optimized versions of camlp4

* Tue Jun 17 2008 Thierry Vignaud <tv@mandriva.org> 3.10.2-2mdv2009.0
+ Revision: 223356
- rebuild

* Sun Mar 02 2008 Stefan van der Eijk <stefan@mandriva.org> 3.10.2-1mdv2008.1
+ Revision: 177752
- 3.10.2, as per bug 37508

* Thu Dec 27 2007 Oden Eriksson <oeriksson@mandriva.com> 3.10.0-5mdv2008.1
+ Revision: 138206
- rebuilt against bdb 4.6.x libs

  + Olivier Blin <blino@mandriva.org>
    - restore BuildRoot

  + Thierry Vignaud <tv@mandriva.org>
    - kill re-definition of %%buildroot on Pixel's request

* Fri Sep 07 2007 Anssi Hannula <anssi@mandriva.org> 3.10.0-4mdv2008.0
+ Revision: 82024
- rebuild for new soname of tcl

* Sun Sep 02 2007 Guillaume Rousse <guillomovitch@mandriva.org> 3.10.0-3mdv2008.0
+ Revision: 78255
- disable automatic dependencies for source package

* Sat Sep 01 2007 Guillaume Rousse <guillomovitch@mandriva.org> 3.10.0-2mdv2008.0
+ Revision: 77694
- add an ocaml-sources subpackage for cduce

* Tue May 29 2007 Pixel <pixel@mandriva.com> 3.10.0-1mdv2008.0
+ Revision: 32452
- new release, 3.10.0
- handle tk 8.5

